/* GSI - general sound interface
 *
 * gsi_commands.h
 *
 * Copyright (C) 1996,1997,1998 W.H.Scholten
 *
 * ** Sound server command specification 0.25 **
 */

/* You can use, redistribute and/or modify this software under the terms of
 * the CGPL version 0.4 or later (see the file LICENSE.CGPL for details)
 */



#ifndef _gsi_commands_h
#  define _gsi_commands_h


//Notation: after a command are denoted the arguments and how many bytes are used for the command and each argument

/*__________________CONTROL COMMANDS___________________*/
//Meta commands
#define SND_OPEN_SERVER             255 /* 1 (+ program name?)for multiple use, or just return an id (byte) which the server needs for each command to know who sent the command */
#define SND_CLOSE_SERVER            254
#define SND_PAUSE_ALL               253 /* 1 pause all sound for the given application */
#define SND_CONTINUE_ALL            252 /* 1 */
//PAUSE_DSP/PAUSE_SYNTH/PAUSE_CD

//Server internal commands
#define SND_SET_DSP_SPEED           247 /* <speed> 1 2 */
#define SND_SET_CHANNELS            246 /* <channels> 1 1 */
#define SND_SET_BYTES               245 /* <type> 1 1 */
#define SND_SET_FRAGSIZE            244 /* <size> 1 1 */

#define SND_GRAB_DSP                243 /* 1 also switches 'autodsp' off */
#define SND_RELEASE_DSP             242 /* 1 also switches 'autodsp' off */

#define SND_AUTO_DSP                241 /* 1 automatically grabs the DSP when it receives a play command, releases it when all sounds have been played; particularly usefull for apps that only want to emit beeps and such */


#define SND_SET_MAX_AUTO_CHANNELS   231 /* <n> 1 1  .. This sets the maximum simultaneous number of playing sounds via channel 0 */


#define SND_DO_CALLBACK             128 /* 'or' with a play command (SND_PLAY - SND_CHANNEL_CONTINUE), also use with SND_SET_CURRENT */
#define SND_CALLBACK_ON             230 /* switches callback on for all commands until a SND_CALLBACK_OFF */
#define SND_CALLBACK_OFF            229 /* switches callback off again */


//Sound dir, sound file, load sounds, free sounds etc.
#define SND_LOAD_SOUND_LIST         223
#define SND_FREE_SOUND_LIST         222 /* 1 must also free sounds that have been loaded */
#define SND_LOAD_LISTED             221
#define SND_FREE_LISTED             220 /* 1 free paths for listed sounds */

#define SND_LOAD_SOUND              219 /* <n> 1 1 (load slot 'n', i.e. on the list) */
#define SND_LOAD_SOUND_FILE         218 /* <snd> <string> 1 1 strlen+1, string gives file path (string ends with a 0) */
#define SND_LOAD_SOUND_DATA         217 /* <snd> <speed> <type> <channels> <data_length> <data> 1 1 4 1 1 4 data_length */
/* possibly use SHM (pipe seems fast enough) */

#define SND_FREE_SOUND              216 /* <snd> 1 1 */

#define SND_SET_SOUND_DIR           215 /* <string> 1 strlen+1, For files in list */
//Free memory used for a sound when it is not playing on any stack/channel (load again when needed)
#define SND_AUTO_FREE_SOUND         214 /* <slot> 1 1 */

//These 5 are not yet implemented:
#define SND_SET_SOUND_TRANSFORM     207 /* ?? */
#define SND_SET_SOUND               206 /* set waveform etc, instead of SND_LOAD_SOUND; + data (wave type(sine, sawtooth, square, ..) , freq, ... ) */
//rename? SND_SET_SOUND_TYPE / SND_SET_SOUND_WAVE
#define SND_SET_STREAM_SIZE         205 /* <size> 1 4, sounds larger than the given stream size are not loaded completely into memory, but loaded 8K at a time, good value is 64K */
//Other useful things?
#define SND_LOCK_SOUNDS             204 /* Don't allow new sounds to be loaded */
#define SND_UNLOCK_SOUNDS           203 /* Allow new sounds to be loaded */



/*__________________PLAY SOUND COMMANDS___________________*/
//Sound playing commands: Change for the new idea: channel 0 is automatic, 1-255 is fixed channels., hence all: +1 byte channel, + 1 byte sound

//Stack playing sounds:
#define SND_PLAY                      0 /* <snd> 1 1 */
//#define SND_PLAY_IF_NOT_PLAYING     0
#define SND_OVERWRITE                 1 /* <snd> 1 1 */
#define SND_REPEAT                    2 /* <snd> 1 1 repeat a sound untill killed */
#define SND_OVERWRITE_REPEAT          3 /* <snd> 1 1 */
#define SND_LOOP                      4 /* <snd> <loopno> 1 1 1, loopno=0 gives repeat forever? */
#define SND_OVERWRITE_LOOP            5 /* <snd> <loopno> 1 1 1, ditto */
#define SND_PLAY_WAVE                 7 /* <wave_type> <freq> <vol> 1 1 2 1 */
//rename SND_PLAY_NOTE ?
//#define SND_NOTE                   10 /* Use wave table: chan, freq, dur */
//#define SND_FREQ                   11 /* for note */
//#define SND_HOLD                    8 /* 'or': to start the sound paused */
//synchronize with other sounds etc? SND_SYNC <num_chan's> <ch 1> <ch 2> ...


//#define SND_KILL_THESE             12 /* <snd> 1 1 (kill all sounds of the given type) */
#define SND_KILL                     13 /* <snd> 1 1 (kill last started slot 'snd') */
#define SND_PAUSE                    14 /* <snd> 1 1 (pause last started slot 'snd') */
#define SND_CONTINUE                 15 /* <snd> 1 1 (ditto) */


#define SND_CHANNEL                  16 /* 'OR' this with above stack commands to get a channel command (ONLY FOR INTERNAL USE) */
//Channel playing sounds:
#define SND_CHANNEL_PLAY             16 /* <ch> <snd> 1 1 1 play a sound on a channel */
#define SND_CHANNEL_REPEAT           17 /* <ch> <snd> 1 1 1 */
#define SND_CHANNEL_OVERWRITE        18 /* <ch> <snd> 1 1 1 */
#define SND_CHANNEL_OVERWRITE_REPEAT 19 /* <ch> <snd> 1 1 1 */
//#define SND_CHANNEL_KILL_THESE     28 /* <snd> 1 1 (kill all sounds of the given type) */
#define SND_CHANNEL_KILL             29 /* <ch> 1 1 */
#define SND_CHANNEL_PAUSE            30 /* <ch> 1 1 */
#define SND_CHANNEL_CONTINUE         31 /* <ch> 1 1 */

//
#define SND_KILLALL                  63 /* 1 */



//Unimplemented:
//Patch loading
#define SND_LOAD_PATCH_DATA          40 /* <bank> <len> <data>  1 1 4 len */
#define SND_LOAD_PATCH_FILE          41 /* <bank> <string>      1 1 strlen+1 */
//Midi loading
#define SND_LOAD_SONG_DATA           42 /* <len> <data>  1 4 len */
#define SND_LOAD_SONG_FILE           43 /* <string>      1 strlen+1 */


/*__________________SOUND MODIFICATION BITS___________________*/
//Bits denoting what effects may/must influence a sound:
#define SND_BIT_CHANNEL_X             1
#define SND_BIT_CHANNEL_DX            2 /* if x then also dx, hence not needed */
#define SND_BIT_CHANNEL_Z             4
#define SND_BIT_CHANNEL_DZ            8
#define SND_BIT_OBSERVER_X           16
#define SND_BIT_OBSERVER_DX          32 /* remove */
#define SND_BIT_OBSERVER_Z           64
#define SND_BIT_OBSERVER_DZ         128 /* remove */
#define SND_BIT_OBSERVER_ANGLE       ??
#define SND_BIT_OBSERVER_DANGLE      ?? /* remove */

#define SND_BIT_CHANNEL_ECHO        256
#define SND_BIT_ALL_ECHO            512 /* echo for all sounds */
#define SND_BIT_CHANNEL_DOPPLER    1024
#define SND_BIT_ALL_DOPPLER        2048 /* doppler for all sounds */
#define SND_BIT_NOISE              4096 /* put some noise on the sound ?? */
#define SND_BIT_ENVELOPE           8192 /* use envelope */

//Of, wat een transformatie bevat!


/*__________________SOUND MODIFICATION COMMANDS___________________*/
#define SND_CURRENT                  32 /* 'OR' with a command to access the last started sound */
//#define SND_SET_CURRENT              SND_CURRENT

#define SND_CHANGE_VOLUME            79 /* <snd> <l> <r> 1 1 1  volume of last slot 'snd' on the stack */
#define SND_CHANGE_CHANNEL_VOLUME    95 /* <ch> <l> <r> 1 1 1  volume of channel ch */
//rename? SND_SET_VOLUME, SND_SET_CHANNEL_VOLUME

//Between brackets: arguments when using SND_CURRENT (i.e. no <slot>):
#define SND_SET_X                    64 // <slot> <x>  1 1 2  { <x>  1 2 }
#define SND_SET_DX                   65 // <slot> <dx> 1 1 1  { <dx> 1 1 }
#define SND_SET_Z                    66 // <slot> <z>  1 1 2  { <z>  1 2 }
#define SND_SET_DZ                   67 // <slot> <dz> 1 1 1  { <dz> 1 1 }
#define SND_SET_Y                    68 // <slot> <y>  1 1 2  { <z>  1 2 }
#define SND_SET_DY                   69 // <slot> <dy> 1 1 1  { <dz> 1 1 }
//with SND_SET_(CHANNEL_)ANGLE also give new radius, and use old radius if r=0?
#define SND_SET_ANGLE                70 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_ANGLE_R              72 // <slot> <angle> <r> 1 1 1 2 { <angle> <r> 1 1 2 }
//This one sets the angle and the distance to the default distance.
#define SND_SET_ANGLE_DEFAULT_R      71 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_PAN                  73 // <slot> <pan position> 1 1 2  -256 gives a sound fully left, 0 centered, 256 fully right. This disables 3d effects for the given sound (doppler, motion)

#define SND_SET_ECHO                 74 // <slot> <dt> <fvolume> 1 1 2 1
#define SND_SET_ECHO_OFF             75 // <slot> 1 1
#define SND_SET_DOPPLER              76 // <slot> <speed> 1 1 2
#define SND_SET_DOPPLER_OFF          77 // <slot> 1 1
#define SND_SET_USE_ENVELOPE         78 // <slot> <env> 1 1 1

//'OR' with SND_CHANNEL to set transforms for CHANNEL sounds
//  SND_SET_* | SND_SET_CHANNEL = SND_SET_CHANNEL_*
#define SND_SET_CHANNEL_X            80 // <ch> <x> 1 1 2     { <x> 1 2  }
#define SND_SET_CHANNEL_DX           81 // <ch> <dx> 1 1 1    { <dx> 1 1 }
#define SND_SET_CHANNEL_Z            82 // <ch> <z> 1 1 2     { <z> 1 2  }
#define SND_SET_CHANNEL_DZ           83 // <ch> <dz> 1 1 1    { <dz> 1 1 }
#define SND_SET_CHANNEL_Y            84 // <ch> <y> 1 1 2     { <z> 1 2  }
#define SND_SET_CHANNEL_DY           85 // <ch> <dy> 1 1 1    { <dz> 1 1 }
//with SND_SET_(CHANNEL_)ANGLE also give new radius, and use old radius if r=0?
#define SND_SET_CHANNEL_ANGLE        86 // <ch> <angle> 1 1 1 { <angle> 1 1 }
//or new:
#define SND_SET_CHANNEL_ANGLE_R      88 // <ch> <angle> <r> 1 1 1 2 { <angle> 1 1 }
//This one sets the angle and the distance to the default distance.
#define SND_SET_CHANNEL_ANGLE_DEFAULT_R      87 // <slot> <angle> 1 1 1 { <angle> 1 1 }
#define SND_SET_CHANNEL_PAN          89 // <ch> <pan position> 1 1 2  -256 gives a sound fully left, 0 centered, 256 fully right. This disables 3d effects on the given channel (doppler, motion)

#define SND_SET_CHANNEL_ECHO         90 // <ch> <dt> <fvolume> 1 1 2 1
#define SND_SET_CHANNEL_ECHO_OFF     91 // <ch> 1 1
#define SND_SET_CHANNEL_DOPPLER      92 // <ch> <speed> 1 1 2
#define SND_SET_CHANNEL_DOPPLER_OFF  93 // <ch> 1 1
#define SND_SET_CHANNEL_ENVELOPE     94 // <ch> <env> 1 1 1





//Observer settings:
#define SND_SET_OX                   96 // <x>
#define SND_SET_ODX                  97 // <dx>
#define SND_SET_OZ                   98 // <z>
#define SND_SET_ODZ                  99 // <dz>
#define SND_SET_OBS_ANGLE           100 // <fi> 1 1
#define SND_SET_OBS_DANGLE          101 // <dfi> 1 1
//angle of a sound is not needed for objects (if a sound has a direction, e.g. a sound cone, then handle that in the main program). The angle above for a sound actually places the object relative to the observer

#define SND_SET_OECHO               102 // <dt> <dvolume>
#define SND_SET_OECHO_OFF           103 //
#define SND_SET_ODOPPLER            104 // <speed>
#define SND_SET_ODOPPLER_OFF        105 //

//Transforms:
#define SND_ENVELOPE                 ?? /* associate an envelope with the sound */
#define SND_MAKE_ENVELOPE            ?? /* 16 envelopes ? */

//#define SND_DEFAULT_VOLUME         ?? /* overall volume */
//#define SND_DEFAULT_CHANNEL_VOLUME ?? /* silly ? */
//#define SND_DEFAULT_AUTO_VOLUME    ?? /* silly ? */
//#define SND_CHANGE_AUTO_VOLUME     ?? /* for the stack sounds, only needed if we don't use channel 0 for the stack */

/*__________________CALL BACK___________________*/
#define GSI_CALLBACK_OK               0
#define GSI_CALLBACK_ERROR            1 /* general error */
#define GSI_CALLBACK_NO_SUCH_FILE     2
#define GSI_CALLBACK_BAD_LIST         3 /* eg sounds that can't be loaded ?? */
#define GSI_CALLBACK_BAD_COMMAND      4
#define GSI_CALLBACK_DSP_BUSY         5
#define GSI_CALLBACK_SYNTH_BUSY       6

//Callback only useful for channel playing, not stack, right?
//#define GSI_CALLBACK_END             32 /* <slot> 1 1 number of the sound that finished */
#define GSI_CALLBACK_CHANNEL_END     33 /* <ch> 1 1 */
#define GSI_CALLBACK_CHANNEL_STARTED 32 /* <ch> 1 1 */
#define GSI_CALLBACK_CHANNEL_ENDALL  34 /* 1 (from a KILLALL command) */


//Dsp mode
#define GSI_8BIT_MULAW     1
#define GSI_8BIT           2
#define GSI_16BIT          3

//Dsp channels
#define GSI_MONO           1
#define GSI_STEREO         2

//Mixer channels
#define GSI_MASTER_VOLUME  0
#define GSI_DSP_VOLUME     1
#define GSI_SYNTH_VOLUME   2
#define GSI_CD_VOLUME      3



/*__________________PROGRAM SIDE C/C++ STRUCTURE___________________*/
// This is just a design which is not used at the moment (e.g. a gsi_play(struct gsi_sound_item *s, int effect_bits) can easily be added to allow starting or changing a sound using such structs. For the effect bits, see above.)
#define byte   unsigned char
#define uint16 short
typedef struct gsi_sound_item {
    byte channel;
    byte type; //sample, wave (4x), midi
    byte sample_no;
    uint16 freq;

    byte volume_l; //primary volume adjustments
    byte volume_r;

    int  stereo_x; //12 bit
    int  stereo_dx; //12 bit
    int  dist_z;  //12 bit
    int  dist_dz; //12 bit

    int  echo; // y/n -> effect_bits?
    int  echo_dt;
    int  doppler; // y/n
    int  doppler_speed;  //conversie= speed/256

    int  stereo_flip; // y/n

    int  envelope; //number, -1=none. volume/freq apart?
//    char *sample_file_name; //niet nodig -> sample_no
    int  effect_flags; //which (observer?) transforms affect the sound?
} gsi_sound_item;
#undef byte
#undef uint16

/* DESCRIPTION
This is only useful for devel work at the moment (very little explanations):

Transformations: should these be set per play command or applied each time when a sample/wave is played?

Sound transforms:
(current 3d values: L=-64, R= +64, DIST=64 )

META:
allow_change: a bit array with permissions that any of the following may be applied:

DIRECT:
   set_volume               B0
   set_noise                B1
   set_echo                 B2
   set_doppler (on/off)     B3 (+ speed conversion factor)

INDIRECT 3D:
   distance (z)             B4
   distance speed (dz)      B5
   stereo pos (x)           B6
   stereo speed (dx)        B7
   observer distance (o_z)  B8
   observer distance speed (o_dz) B9
   observer stereo pos (o_x)      B10
   observer stereo pos speed (o_dx) B11

   observer_flip (inv_x)    B12 //gecombineerd met volume setting geeft dit 360 graden rotatie vd waarnemer.

   observer_angle           B13

+ envelope + sound source motion (z+ stereo) + flip (3d, behind observer)

Sounds moving with observer (sounds in a car eg) must not be affected by observer motion. Also information sounds not related to game action (warning beeps).

internal server stuff:
   room_echo, room_echo_dt,
   observer_x, observer_dx, observer_z, observer_dz
   observer_doppler, observer_doppler_speed.
   */

#endif /* _gsi_commands_h */
