/* GSI - general sound interface
 *
 * gsi_server.c - native OSS shared sound server - sept 1996, may/dec 1997, march 1998
 *
 * Copyright (C) 1996,1997,1998 W.H.Scholten
 *
 * This server (currently a dspserver only) is probably Linux/FreeBSD specific
 */

/* This software is licensed under the CGPL version 0.4 or later
 * (see the file LICENSE.CGPL for details)
 */


// .au/.snd file format:
//----------------------------------------------
//  ".snd"  START
//  LENGTH  TYPE
//  FREQ    CHANNELS
//  <DESCRIPTION>    (text)
//  <DATA>           (starts at START, length=LENGTH)
//----------------------------------------------
//   All are 32bit MSB-ints
//   TYPE: 1 = mu-law compressed 8bit, (GSI_8BIT_MULAW)
//         2 = uncompressed 8bit,      (GSI_8BIT)
//         3 = 16bit uncompressed MSB  (GSI_16BIT)
//   CHANNELS : 1 for mono, 2 for stereo.

// internal use :sound type: 0=8bit mono, 1=16 bit mono, 2=8 bit stereo, 3=16 bit stereo


/*__________________INCLUDES___________________*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#ifdef __linux__
#  include <linux/soundcard.h>
#endif
#ifdef __FreeBSD__
#  include <machine/soundcard.h>
#  include <sys/errno.h>
#endif

#include <sys/time.h>
#include <signal.h>
#include <string.h>
#include <sys/stat.h>

#include <math.h>

#include <gsi/mytypes.h>
//#define DEBUG
#include <gsi/debug.h>
#define _SNDSERVER_
#include <gsi/gsi_commands.h>
#include <gsi/gsi_server_version.h>
//#define NO_EFFECTS

/*__________________DEFINITIES___________________*/
//To disable speed conversion stuff etc (fastest):
//#define OLD_PLAYER

#ifdef DEBUG
#  define SEGFAULT_HANDLER
#endif

#define MAX_SOUNDS           256 /* Maximum number of sounds loaded (slots) */
//These two should go to a separate config file:
#define MAX_STATIC_CHANNELS   64 /* Maximum number of simultaneous sounds on channels (the maximum of this value is 256) */
#define MAX_DYNAMIC_CHANNELS  32 /* Maximum number of simultaneous sounds dynamically allocated on a stack (the maximum of this value is only restricted by memory) */
#define MAX_CHANNELS        (MAX_STATIC_CHANNELS + MAX_DYNAMIC_CHANNELS)

#define MAX_STATUS_QUEUE_LEN 256

#define min(x,y) ((x)>(y) ? (y): (x))
#define max(x,y) ((x)<(y) ? (y): (x))



/*__________________VARIABELEN___________________*/
char *server_name;
char *dspdev;
/* ** DSP settings (init's are only used for set_sound_conversion_of(.) as that also gets called before the DSP is initialised. FIX that) */
int  dsp_server_speed=8000;
int  dsp_server_type=GSI_8BIT;
int  dsp_server_log2_frag=8;
int  dsp_server_fragsize=256; /* samples=fragsize/channels/bytes */
int  dsp_server_channels=1;

int  dsp_server_samples; // number of samples that can be played in the fragsize
int  dsp_server_bps; /* bytes per sample, 1 or 2 */

int  autorelease_dsp=False;
	/* If true, the DSP is released when no sounds are playing, it will be
	 * grabbed again when a 'play sound' command is received */
int  do_callback=False;
	/* If true, callbacks will be sent whenever a sound is started or killed
	 * on a channel (not the stack) */

/* ** Decoding the current command from bytes read from the pipe (async mode) */
byte command_size;       // 0= no command processed
byte command;            // if command_size>0 this is the current command
byte command_data[32];   // all further bytes from a command
//Note: commands that send a lot of data (e.g. SND_LOAD_SOUND_DATA) will not be handled in async mode.

//Should these two be set also by sound modify commands? :
int  last_sound_stack;   // stack no of last started sound (OLD:stack no of sound on channel 0)
int  last_sound_channel;
	/* channel of last command (for immediately following update commands),
	 * -1 if none (all done playing or a kill command issued) */
int  channel_used [MAX_STATIC_CHANNELS];
	/* simple linear list */

/* ** various */
int  sinus_tabel[1024];  //f or wave playing and observer rotation

char sounds_dir[256];
int  num_sounds=0;       /* = 1+ highest number of sound slots used */


/* ** Sounds loaded into memory */
int  sounds_loaded;
	/* have the sounds from the sound list been loaded? */
char *SOUNDFILENAME [MAX_SOUNDS];
int8 *sound_buffer [MAX_SOUNDS]; //the actual sound data
int  sound_size [MAX_SOUNDS];
	/* bytes, >>should become samples<< ? (16/8 bit..) */
int  sound_speed [MAX_SOUNDS];
int  sound_type [MAX_SOUNDS];     /* 8/16bit, 8 bit mu-law */
int  sound_channels [MAX_SOUNDS]; /* mono, stereo */
int  sound_conversion [MAX_SOUNDS], sound_conversion2[ MAX_SOUNDS ];
	/* conversion from sound to dsp speed (will be replaced by RL algorithm) */
//int  sound_flags[MAX_SOUNDS];
byte sound_from_list [MAX_SOUNDS];
	/* ??? is it a sound from the list or separately loaded? */

int  max_stack=MAX_DYNAMIC_CHANNELS; /* max number of simultaneous sounds via the stack, reduce with SND_SET_MAX_AUTO_CHANNELS to limit the maximum CPU use */


/* ** Data to control sound effects, speed conversion etc */
short sound_volume [MAX_CHANNELS];
int  sound_x [MAX_CHANNELS];
int  sound_dx [MAX_CHANNELS];
int  sound_z [MAX_CHANNELS];
int  sound_dz [MAX_CHANNELS];
int  sound_y [MAX_CHANNELS];
int  sound_dy [MAX_CHANNELS];

//int  sound_echo [MAX_CHANNELS]; // obsolete?
short sound_echo_dt [MAX_CHANNELS]; // dt*32
byte sound_echo_fvol [MAX_CHANNELS];
	/* fractional volume change, should be 1/(8R^3) where R is distance to the
	 * bounce point => log2: -3 -3* 2log(R) ? => dvol? */
short  sound_doppler [MAX_CHANNELS]; // speed change

int  observer_x = 0,  observer_dx = 0;
int  observer_z = 64, observer_dz = 0;
int  observer_angle = 0;
int  observer_echo  = 0;
int  observer_doppler = 0;

/* ** Current state of sounds playing */
int  playing [MAX_CHANNELS];   /* Sound file numbers currently being played */
int  position [MAX_CHANNELS];  /* Current position in file for each sound */
int  repeat [MAX_CHANNELS];	   /* 0=no repeat if at end of file */
int  playnum = 0;              /* Number of sounds currently being played on the stack */
int  channel_playnum = 0;      /* Number of sounds currently being played on channels */


/* ** The sound mixer buffers */
byte final [2048];
int  premix [512];


/* ** Following should be local variables: */
int  sound_volume_L, sound_volume_R; //stereo position volume factors
//int  sound_volume_L_L, sound_volume_L_R; //stereo position volume factors
//int  sound_volume_R_L, sound_volume_R_R; //stereo position volume factors

//For callbacks:
byte status_queue[MAX_STATUS_QUEUE_LEN];
int  status_queue_len;

/*__________________FUNCTIE PROTOTYPES___________________*/
void  snd_server();
int   setup_dsp(char *dspdev, int speed, int fragsize, int channels, int type);
void  play_fragment(int slot);

int   init_sounds_file(char *sndsfile_name);
void  read_soundsfile_sounds();
int   read_sound(int slot, char *file);
void  unload_sounds_file();
void  unload_sound(int slot);
void  init_sample(int slot, int speed, int type, int channels, int size);

void  set_sound_conversion(); /* all slots */
void  set_sound_conversion_of(int slot);

void  status_send_byte(int b);
void  status_flush();

void  make_sinus();

int   int_msb(int fd); /* read MSB int from file */

void  get_block(byte *data, int len);

byte  get_byte();   // rename read_pipe_byte/read_byte_from_pipe ?
int16 get_int16();
int32 get_int32();
char *get_string(); // Read a string from pipe (for which storage is malloc'ed)

void  quit (int i);
void  segfault(int msg);


/*__________________PROGRAMMA___________________*/

int main (int argc, char **argv) {
#ifdef SEGFAULT_HANDLER
	signal(SIGSEGV, segfault);
#endif
	signal(SIGTERM, quit);

	if (argc != 2) {
		printf (SOUND_SERVER_VERSION"\n");
		exit (0);
	}

		/* Call the plumber to unblock the 'in' pipe! */
	fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);

	server_name=argv[0];
	dspdev="/dev/dsp";

	snd_server();
}


/* Setup DSP:
 *	 opens device (dspdev; usually /dev/dsp)
 *	 sets fragment size ( 2 fragments of size 'fragsize').
 *	 sets speed ('speed')
 *	 sets mono/stereo ('channels')
 *	 sets sample size; 8 bit/16 bit ('type') */
int setup_dsp (char *dspdev, int speed, int log2_fragsize, int channels, int type) {
	int dsp, value;
		//Sample size: 8 bit or 16 bit (mu-law not supported yet)
	int format;
	int snd_formats[]={ AFMT_MU_LAW, AFMT_U8, AFMT_S16_LE };

	DEB1(fprt(stderr, "setup dsp\n"));

	dsp = open(dspdev, O_WRONLY);
	if (dsp < 1) {
		if (errno==EBUSY)
			perror("OPEN DSP");
		else
			fprintf (stderr, "%s: Couldn't open DSP %s\n", server_name, dspdev);
		return -1;
	}
	dsp_server_speed=speed;
	dsp_server_log2_frag=log2_fragsize;
	dsp_server_channels=channels;
	dsp_server_type=type;

		//Fragment size
	DEB1(fprt(stderr, "%s: setting fragment to 2^%d\n", server_name, log2_fragsize));
	value=0x00020000 | log2_fragsize; /* 2 fragments of size fragsize */ 
	ioctl(dsp, SNDCTL_DSP_SETFRAGMENT, &value);

		//Channels
		//if (ioctl(dsp, SNDCTL_DSP_CHANNELS, &dsp_server_channels)==-1) {
	value = dsp_server_channels-1; //0=mono, 1=stereo
	if (ioctl(dsp, SNDCTL_DSP_STEREO, &value)==-1) {
			//Fatal error.
		perror("SNDCTL_DSP_STEREO");
		exit(-1);
	}
	if (dsp_server_channels==2 && value!=1) {
		fprintf(stderr, "DSP device doesn't support stereo, using mono.\n");
		dsp_server_channels=1;
	}
	DEB1(fprt(stderr, "channels=%d\n", value+1));

		//Sample size: 8 bit or 16 bit (mu-law not supported yet)
	format = snd_formats[dsp_server_type-1];

	if (ioctl(dsp, SNDCTL_DSP_SETFMT, &format)==-1) {
			/* Fatal error */
		perror("SNDCTL_DSP_SETFMT");
		exit(-1);
	}

	if (format != snd_formats[dsp_server_type-1]) {
		fprintf(stderr, "The DSP device doesn't support the requested audio format\n");
			//Now try 8 bit instead of 16 or reverse.
		dsp_server_type ^=1;
		format = snd_formats[dsp_server_type-1];

		if (ioctl(dsp, SNDCTL_DSP_SETFMT, &format)==-1) {
				//Fatal error
			perror("SNDCTL_DSP_SETFMT");
			exit(-1);
		}
	}

		//Speed
	value=speed;
	if (ioctl(dsp, SNDCTL_DSP_SPEED, &value)) {
		fprintf (stderr, "%s: Couldn't set DSP rate\n", server_name);
	}
	DEB1(fprt(stderr, "DSP speed set at %d\n", value));
	dsp_server_speed=value;

		//Fragment size
	dsp_server_fragsize = 0;
	ioctl(dsp, SNDCTL_DSP_GETBLKSIZE, &dsp_server_fragsize);
	if (!dsp_server_fragsize) {
		fprintf (stderr, "%s: Couldn't set fragment size.\n", server_name);
		exit(0);
	}
	DEB1(fprt(stderr, "server fragsize=%d\n", dsp_server_fragsize));

	dsp_server_bps= (dsp_server_type & 1)+1;

	dsp_server_samples= dsp_server_fragsize/(dsp_server_bps*dsp_server_channels);



//   final= (byte *) malloc(dsp_server_fragsize);
//   premix= (int *) malloc(dsp_server_samples);


	DEB1(fprt(stderr, "server bps=%d, samples=%d, channels=%d, speed=%d\n", dsp_server_bps, dsp_server_samples, dsp_server_channels, dsp_server_speed));

	return dsp;
}

#if 0
void auto_dsp_update() {
	if (autorelease_dsp) {
		if (playnum>0 || channel_playnum>0) { /* we want to use the DSP */
			if (dsp<0)
				dsp=setup_dsp(dspdev, new_dsp_server_speed, new_dsp_server_log2_frag, new_dsp_server_channels, new_dsp_server_type);
		}
		else { /*we don't need the DSP any more */
			if (dsp>0) { close(dsp); dsp=-1; }
		}
	}
}
#endif

///////////// Main loop ///////////////
void snd_server () {
	int  dsp;
	boolean paused=False;

	char *sample_position;

	int  i, j, k, p;
	int  command, data;
	byte b;// pipe data

		//Defaults
	int new_dsp_server_speed=8000;
	int new_dsp_server_log2_frag=8;
	int new_dsp_server_channels=1; //mono
	int new_dsp_server_type=2; //default 8 bit


	dsp=-1; //Init, dsp not opened.

	make_sinus(); //for notes, observer rotation etc.

	for (i=0; i<MAX_STATIC_CHANNELS; i++) 
		playing[i+MAX_DYNAMIC_CHANNELS]=-1;

	for(;;) {
			// + ------------------------------------------- +
		for (;;) { /* loop to get all sound commands */
			i=read(STDIN_FILENO,&b,sizeof(b));
			command=b; //=> if (command_size==0) command=b;
			DEB1(if (i>0) fprt(stderr, "GSI_SERVER: command %d\n", b));

			if(i==0)  {
					/* EOF on the pipe means the parent has closed its end */
				DEB1(fprt(stderr, "%s: shutting down\n", server_name));
				kill(getpid(), SIGTERM);
			}
			if (i==-1) {
				if (dsp !=-1 && !paused)
					break;
				else {
					fd_set read_fd;
					FD_ZERO(&read_fd);
					FD_SET(0, &read_fd);
					select(1, &read_fd, NULL, NULL, NULL);
					continue;
				}
			}
			if(i!=-1)  {  /* there was something in the pipe */
				DEB2(fprt(stderr, "read a %d from pipe\n",(int) k));

				if ( dsp <0 ) {
					if (command==SND_GRAB_DSP) {
						autorelease_dsp=False;
						dsp=setup_dsp(dspdev, new_dsp_server_speed, new_dsp_server_log2_frag, new_dsp_server_channels, new_dsp_server_type); /* all other commands ignored */
//ASYNC:				command_size=0;
						if ( dsp <0 )
							fprintf(stderr, "failed to grab dsp!\n");
						else {
							DEB1(fprintf(stderr, "dsp grabbed\n"));
						}
					}
					if (dsp >-1) { /* we get here if the DSP was grabbed */
						if (!sounds_loaded)
							read_soundsfile_sounds();
						set_sound_conversion();
					}
				}
				else {
					if (command==SND_RELEASE_DSP) {
						autorelease_dsp=False;
						DEB1(fprt(stderr, "dsp released\n"));
//AYNC:					command_size=0;
						close(dsp); dsp=-1; continue;
					}
				}
					// + ------------------------------------------- +
					//Command decoding:
//ASYNC:		command_data[sound_size-1]=get_byte(); /* This is WRONG, fix */
//ASYNC:		DEB1(fprt(stderr, "GSI_SERVER: command=%d, data=%d\n", command, command_data));
				switch(command) {
				case SND_GRAB_DSP:
				case SND_RELEASE_DSP:
					autorelease_dsp=False;
					break;
				case SND_AUTO_DSP:
					autorelease_dsp=True;
					break;
				case SND_PAUSE_ALL:
					paused=True;
					continue;
				case SND_CONTINUE_ALL:
					paused=False;
					continue;
				case SND_OPEN_SERVER:
					break;
				case SND_CLOSE_SERVER:
					exit(1);
				case SND_SET_DSP_SPEED:
//command_data[sound_size++ -1]=get_byte();
//ASYNC:			if (command_size<3) continue;
//ASYNC:			new_dsp_server_speed=command_data[0] | (command_data[1]<<8);
//ASYNC:			command_size=0;
					DEB1(fprt(stderr, "DSP_SERVER: set dsp speed\n"));
					new_dsp_server_speed = (int) (uint16) get_int16();
					DEB1(fprt(stderr, "           set to %d\n", new_dsp_server_speed));

					break;
				case SND_SET_CHANNELS:
					new_dsp_server_channels = (int) get_byte();
//ASYNC:			if (command_size<1) continue;
//ASYNC:			new_dsp_server_channels=command_data[0];
//ASYNC:			command_size=0;
					break;
				case SND_SET_BYTES: //should be SND_SET_TYPE
					new_dsp_server_type = (int) get_byte();
//ASYNC:			if (command_size<1) continue;
//ASYNC:			new_dsp_server_type=command_data[0];
//ASYNC:			command_size=0;
					break;
				case SND_SET_FRAGSIZE:
					new_dsp_server_log2_frag = get_byte();
//ASYNC:			if (command_size<1) continue;
//ASYNC:			new_dsp_server_log2_frag = command_data[0];
//ASYNC:			command_size=0;
					break;

				case SND_SET_MAX_AUTO_CHANNELS:
					max_stack=min(get_byte(), MAX_DYNAMIC_CHANNELS);
					break;

				case SND_LOAD_SOUND_LIST:
					DEB1(fprt(stderr, "DSP_SERVER: SND_LOAD_SOUND_LIST\n"));
				{
					char *name=get_string();
					init_sounds_file(name);
					free(name);
					break;
				}
				case SND_FREE_SOUND_LIST:
				{
					unload_sounds_file();
					break;
				}
				case SND_LOAD_LISTED:
				{
					read_soundsfile_sounds();
					break;
				}
				case SND_FREE_LISTED:
				{
					unload_sounds_file(); //should not release the list file.
					break;
				}
				case SND_LOAD_SOUND:
				{
					int  slot=get_byte();
					if (SOUNDFILENAME[slot]!=NULL)
						read_sound(slot, SOUNDFILENAME[slot]);
				}
					break;
				case SND_LOAD_SOUND_FILE:
					DEB1(fprt(stderr, "GSI_SERVER: SND_LOAD_SOUND_FILE\n"));
					{
							//Always SYNC mode here
					int  slot=get_byte();
					char *name=get_string();
					if (slot < /*max_slot*/ MAX_SOUNDS) {
						num_sounds=max(num_sounds, slot+1);
						read_sound(slot, name);
					}
					free(name);
					break;
					}
				case SND_LOAD_SOUND_DATA:
					DEB1(fprt(stderr, "GSI_SERVER: SND_LOAD_SOUND_DATA\n"));
					{
							//Always SYNC mode here
					int  l;
					int  slot=get_byte();
					int  sample_speed=get_int32();
					int  sample_type=get_byte();
					int  sample_channels=get_byte();
					int  sample_len=get_int32();
					if (slot < /*max_slot*/ MAX_SOUNDS) {
//						if (sound_buffer[slot]!=NULL) {
//							free(sound_buffer[slot]);
//							sound_buffer[slot] = NULL;
//						}
						unload_sound(slot);

						sound_buffer[slot]=(byte *) malloc(sample_len * sample_channels * ((sample_type+1)>>1));
						get_block(sound_buffer[slot], sample_len);

						num_sounds=max(num_sounds, slot+1);
						init_sample(slot, sample_speed, sample_type, sample_channels, sample_len);
					}
					else for (l=0; l<sample_len; l++) get_byte(); //ignore all sent
					break;
					}
				case SND_FREE_SOUND:
					DEB1(fprt(stderr, "GSI_SERVER: SND_FREE_SOUND\n"));
//ASYNC:			if (command_size<1) continue;
					{
//ASYNC:				slot = command_data[0];
//ASYNC:				command_size=0;
						int  slot=get_byte();
						unload_sound(slot);
					}
					break;
				case SND_SET_SOUND_DIR:
					DEB1(fprt(stderr, "GSI_SERVER: SET_SOUND_DIR\n"));
					{ //SYNC
					char *name=get_string();
					DEB1(fprt(stderr, "  sounds_dir=%s\n", name));
					strncpy(sounds_dir, name, 255);
					sounds_dir[255]=0;
					free(name);
					break;
					}
				case SND_CALLBACK_ON:
					DEB1(fprt(stderr, "GSI_SERVER: callback on\n"));
					do_callback=True; break;
				case SND_CALLBACK_OFF:
					DEB1(fprt(stderr, "GSI_SERVER: callback off\n"));
					do_callback=False; break;

				default:
//					if (dsp ==-1 || paused) continue;

					if (command == SND_KILLALL) {
						int  pc;
						DEB1(fprt(stderr, "KILLALL\n"));
						playnum=0;
						for (pc=0; pc<channel_playnum; pc++) 
							playing[channel_used[pc]+MAX_DYNAMIC_CHANNELS]=-1;
						channel_playnum=0;
						if (do_callback) status_send_byte(GSI_CALLBACK_CHANNEL_ENDALL);
						continue;
					}
						// + ------------------------------------------- +
					if (command <32) { //playing commands
							// Get the channel/sound number
//ASYNC:				p=command_data[0]; /* sound to be played */
							//get <ch> if command >=16
						int play_channel;
						p=get_byte(); // sound slot
						play_channel=p; // it may be the channel though..
						if (command >=16 && command <16+12) {
							p=get_byte(); // this is the sound slot
							DEB1(fprt(stderr, "channel stuff: play_channel=%d, slot=%d\n", play_channel, p));
						}

							// + ------------------------------------------- +
						if (command<16) { /* Stack channel stuff */
						if( command == SND_KILL )  {
							DEB1(fprt(stderr, "terminating sound %d\n", (int) p));
							for(i=playnum-1; i>=0; i--) {
								if ( playing[i]==p)  {
									if (last_sound_stack==i) last_sound_stack=-1;
									if  ( i<playnum-1 ) {
										memmove(repeat+i, repeat+i+1,
												(playnum-i-1)*sizeof(int));
										memmove(playing+i, playing+i+1,
												(playnum-i-1)*sizeof(int));
										memmove(position+i, position+i+1,
												(playnum-i-1)*sizeof(int));
										memmove(sound_x+i, sound_x+i+1,
												(playnum-i-1)*sizeof(int));
										memmove(sound_dx+i, sound_dx+i+1,
												(playnum-i-1)*sizeof(int));
									}
									playnum--;
									break;
								}
							}
							DEB1(if (i<0) fprintf(stderr, "  not found!\n"));
						} else { //sound play/overwrite/loop commands
							if ( playnum< max_stack/*MAX_CHANNELS*/ )  {
									//Load sound if necessary (and if possible)
								if ( sound_size[(int)p]==0 ) read_sound(p, SOUNDFILENAME[p]);
								if ( sound_size[(int)p]>0 )  {
									j=playnum;
									if (command == SND_OVERWRITE) {
										for (i=playnum-1; i>=0; i--)
											if (playing[i]==p) { j=i; playnum--; break; /*ugly*/ }
									}
									repeat   [j] = (command == SND_REPEAT);
									position [j] = 0;
									playing  [j] = p;
									playnum++;
										//Now set defaults
									sound_volume[j] = 256;
									sound_x  [j] = 0;
									sound_dx [j] = 0;
									sound_z  [j] = 64;
									sound_dz [j] = 0;
									sound_echo_dt[j]   = 0;
									sound_echo_fvol[j] = 255;
									sound_doppler[j]   = 0;
									last_sound_stack=j;
//OBSOLETE:					For SND_CURRENT modify commands: set the last_id
//OBSOLETE:					last_id=j*256+0; //channel 0, sound j
								}
								DEB1(fprt(stderr, "sound %d added to play-queue at %d\n", p, playnum-1));
							}
						}
						} /* ^ commands 0-15 */
							// + ------------------------------------------- +
						else { /* command 16-31 */
/////////////////// STATIC CHANNELS ////////////////////////
						if( command == SND_CHANNEL_KILL )  {
							DEB1(fprt(stderr, "terminating sound on channel %d\n", (int) play_channel));
							for(i=channel_playnum-1; i>=0; i--) {
								if ( channel_used[i]==play_channel)  {
									DEB1(fprintf(stderr, "found sound, listno=%d\n", i));
									if (last_sound_stack==i) last_sound_stack=-1;
									if  ( i<channel_playnum-1 ) {
										memmove(channel_used+i, channel_used+i+1,
												(channel_playnum-i-1)*sizeof(int));
									}
									playing[play_channel+MAX_DYNAMIC_CHANNELS]=-1;
									if (do_callback) { status_send_byte(GSI_CALLBACK_CHANNEL_END); status_send_byte(play_channel); DEB1(fprintf(stderr, "callback in queue\n")); }
									channel_playnum--;
									break;
								}
							}
							DEB1(if (i<0) fprintf(stderr, "  not found !\n"));
							continue;
						} else { //sound play/overwrite/loop commands
							if ( play_channel< MAX_STATIC_CHANNELS )  {
									//Load sound if necessary (and if possible)
								if ( sound_size[(int)p]==0 ) read_sound(p, SOUNDFILENAME[p]);
								if ( sound_size[(int)p]>0 )  {
									j=MAX_DYNAMIC_CHANNELS+play_channel;
									if (playing[play_channel+MAX_DYNAMIC_CHANNELS]>=0) {
											//already a sound playing, overwrite?
										if (!(command == SND_OVERWRITE || command == SND_OVERWRITE_REPEAT)) {
											continue;
										}
									}
									else { //new channel
										channel_used[channel_playnum]=play_channel;
										channel_playnum++;
										if (do_callback) {
											status_send_byte(GSI_CALLBACK_CHANNEL_STARTED);
											status_send_byte(play_channel);
										}
									}
									repeat   [j] = (command == SND_CHANNEL_REPEAT);
									position [j] = 0;
									playing  [j] = p;
										//Now set defaults
									sound_volume[j] = 256;
									sound_x  [j] = 0;
									sound_dx [j] = 0;
									sound_z  [j] = 64;
									sound_dz [j] = 0;
									sound_echo_dt[j]   = 0;
									sound_echo_fvol[j] = 255;
									sound_doppler[j]   = 0;

									last_sound_channel=play_channel;
//OBSOLETE:					For SND_CURRENT modify commands: set the last_id
//OBSOLETE:					last_id=j*256+0; //channel 0, sound j
								}
								DEB1(fprt(stderr, "sound %d started on channel %d (channel_used number %d)\n", p, play_channel, channel_playnum));
							}
						}
						} /* ^ commands 16-31 */
					} /* ^ commands 0-31 */

						// + ------------------------------------------- +
						/* Now handle all other commands: sound modifications,
						 * stereo position, distance, doppler etc. */
					else {
						int  command2=command;
						int  data2=command_data[0];
							command2 &=~(SND_CURRENT | SND_CHANNEL);

						if (command & SND_CURRENT) {
							DEB1(fprt(stderr, "GSI_SERVER: last stack=%d, last_channel=%d\n", last_sound_stack, last_sound_channel));
							if (command & SND_CHANNEL)
								j=last_sound_channel+MAX_DYNAMIC_CHANNELS;
							else
								j=last_sound_stack;
						}
						else {
							p=get_byte();
							if (command & SND_CHANNEL)
								j=p+MAX_DYNAMIC_CHANNELS;
							else { //find the stack entry that we want to change
								int pc;
								for (pc=playnum; pc>=0; pc--) {
									if (playing[pc]==p) {
										j=pc;
										break;
									}
								}
								if (pc<0) continue; //slot p is not being played on any stack channel
							}
						}

						DEB1(fprt(stderr, "GSI_SERVER: command=%d, command2=%d, data2=%d, j=%d\n", command, command2, data2, j));
//ASYNC:			sound_flags[j]= /* 1<< */ command2;
#define EAR_POS   64
#define EAR_DIST  64
//To access STATIC channels: j =j+ MAX_DYNAMICS_CHANNELS;
						switch (command2) {
						case SND_SET_X:
							DEB1(fprt(stderr, "SND_X\n"));
//ASYNC:				sound_x[j] = data2;
//ASYNC:				command_size=0;
							sound_x[j] = get_int16();
							break;
						case SND_SET_DX:
							DEB1(fprt(stderr, "SND_DX\n"));
//ASYNC:				sound_dx[j] = data2;
							sound_dx[j] = (int) (int8) get_byte();
							break;
						case SND_SET_Z:
							DEB1(fprt(stderr, "SND_Z\n"));
//ASYNC:				sound_z[j] = data2;
							sound_z[j] = get_int16();
							break;
						case SND_SET_DZ:
							DEB1(fprt(stderr, "SND_DZ\n"));
//ASYNC:				sound_dz[j] = data2;
							sound_dz[j] = (int) (int8) get_byte();
							break;
						case SND_SET_Y:
							DEB1(fprt(stderr, "SND_Y\n"));
//ASYNC:				sound_y[j] = data2;
							sound_y[j] = get_int16();
							break;
						case SND_SET_DY:
							DEB1(fprt(stderr, "SND_DY\n"));
//ASYNC:				sound_dy[j] = data2;
							sound_dy[j] = (int) (int8) get_byte();
							break;
						case SND_SET_ANGLE:
							DEB1(fprt(stderr, "SND_SET_ANGLE\n"));
//ASYNC:				angle = data2;
							{
								int angle = get_byte();
								int x=sound_x[j];
								int z=sound_z[j];
								int radius;
								int cos_angle, sin_angle;

								cos_angle=sinus_tabel[((64-angle)&255)<<2];
								sin_angle=sinus_tabel[angle<<2];
								if (x==0) radius=z; else radius=sqrt(x*x+z*z); //no slow sqrt when the sound is in start pos.
//use 32bit integer sqrt routine.
								sound_x[j] = (radius*cos_angle)>>16;
								sound_z[j] = (radius*sin_angle)>>16;
								break;
							}

						case SND_SET_ANGLE_R:
							DEB1(fprt(stderr, "SND_SET_ANGLE_R\n"));
//ASYNC:				angle = data2;
							{
								int angle = get_byte();
								int radius= get_int16();
								int cos_angle, sin_angle;

								cos_angle=sinus_tabel[((64-angle)&255)<<2];
								sin_angle=sinus_tabel[angle<<2];
								sound_x[j] = (radius*cos_angle)>>16;
								sound_z[j] = (radius*sin_angle)>>16;
								break;
							}

						case SND_SET_ANGLE_DEFAULT_R:
								//sets the angle and standard radius (i.e. no sqrt needed)
							DEB1(fprt(stderr, "SND_SET_ANGLE_DEFAULT_R\n"));
//ASYNC:				angle = data2;
							{
								int angle = get_byte();
								int cos_angle, sin_angle;
								cos_angle=sinus_tabel[((64-angle)&255)<<2];
								sin_angle=sinus_tabel[angle<<2];
								sound_x[j] = (64*cos_angle)>>16;
								sound_z[j] = (64*sin_angle)>>16;
								break;
							}

						case SND_CHANGE_VOLUME:
							DEB1(fprt(stderr, "SND_CHANGE_VOLUME\n"));
							{
							sound_volume[j]= get_byte ();
							if (sound_volume[j]>0) sound_volume[j]++;
							break;
							}

						case SND_SET_OX:
							DEB1(fprt(stderr, "SND_OX\n"));
//ASYNC:				observer_x = data2;
							observer_x = get_int16();
							break;
						case SND_SET_ODX:
							DEB1(fprt(stderr, "SND_ODX\n"));
//ASYNC:				observer_dx = data2;
							observer_dx = (int) (int8) get_byte();
							break;
						case SND_SET_OZ:
							DEB1(fprt(stderr, "SND_OZ\n"));
//ASYNC:				observer_z = data2;
							observer_z = get_int16();
							break;
						case SND_SET_ODZ:
							DEB1(fprt(stderr, "SND_ODZ\n"));
//ASYNC:				observer_dz = data2;
							observer_dz = (int) (int8) get_byte();
							break;
						case SND_SET_OBS_ANGLE:
							DEB1(fprt(stderr, "SND_OBS_ANGLE\n"));
//ASYNC:				observer_angle = data2;
							observer_angle = (int) (int8) get_byte();
							break;
						case SND_SET_ECHO:
							DEB1(fprt(stderr, "SND_ECHO\n"));
//ASYNC:				if (command_size<3) continue;
//ASYNC:				sound_echo_dt[j] = data2;
//ASYNC:				sound_echo_dt[j] = command_data[1];
							sound_echo_dt[j] = (int) (uint) get_int16();
							sound_echo_fvol[j] = get_byte();
							break;
						case SND_SET_ECHO_OFF:
							DEB1(fprt(stderr, "SND_ECHO_OFF\n"));
//ASYNC:				sound_echo_dt[j] = 0;
							sound_echo_dt[j] = 0;
							break;
						case SND_SET_DOPPLER:
							DEB1(fprt(stderr, "SND_DOPPLER\n"));
//ASYNC:				sound_doppler[j] = data2;
							sound_doppler[j] = get_int16(); //should calc conv factors from this (this is the speed in terms of sound velocity).
// f'=f *Vg/(Vg+Vobj). the same factor is used for the volume change (number of J/(m^2 s) increases/decreases with the frequency)
							DEB1(fprt(stderr, "ch %d, doppler=%d\n", j, sound_doppler[j]));
							break;
						case SND_SET_DOPPLER_OFF:
							DEB1(fprt(stderr, "SND_DOPPLER_OFF\n"));
//for all channels or just one?
							sound_doppler[j] = 0;
							break;
						case SND_SET_ODOPPLER:
							DEB1(fprt(stderr, "SND_ODOPPLER\n"));
//ASYNC:				observer_doppler = data2;
							observer_doppler = get_int16(); //should calc conv factors from this (calculate speed_conversion_total/speed_conversion2_total)
							break;
						case SND_SET_ODOPPLER_OFF:
							DEB1(fprt(stderr, "SND_ODOPPLER_OFF\n"));
//ASYNC:				observer_doppler = 0;
							observer_doppler = 0;
							break;
						default:
							fprintf(stderr, "Bad dsp server command %d\n", command);
						} //switch (command2)
					}

					if (command <=32 && autorelease_dsp) {
						if (playnum>0 /*|| static_playnum>0 */) { /* we want to use the DSP */
							if (dsp<0)
								dsp=setup_dsp(dspdev, new_dsp_server_speed, new_dsp_server_log2_frag, new_dsp_server_channels, new_dsp_server_type);
						}
						else { /*we don't need the DSP any more */
							if (dsp>0) { close(dsp); dsp=-1; }
						}
					}

				} //switch (command)
			} /* if (i!=-1) */

		} /* end of loop to get all sound commands */


			// -- + -------------------------------------------------------- +
			//send all callback info (only so much that we don't need to wait)
		status_flush();

//		check available space.
//		int space;
//		ioctl(read_fd, SNDCTL_DSP_GET_OSPACE, &space);
//		if (space <dsp_server_fragsize) continue;

		if (!(playnum | channel_playnum) && autorelease_dsp && dsp>-1) {
			close(dsp); dsp=-1;
			DEB1(fprt(stderr, "autorelease: dsp = -1\n"));
			continue;
		}
//Make this an assert:
		if (dsp<1) { fprintf(stderr, "dsp=-1\n"); exit(1); }

			//Now play all the active sounds
		if (playnum || channel_playnum)  {
				/* ** Mix the sounds into the final buffer */
			memset(premix, 0, sizeof(premix));

			for(i=0; i<playnum; i++)  {
				play_fragment(i);
			}
			for(i=0; i<channel_playnum; i++)  {
				play_fragment(channel_used[i]+MAX_DYNAMIC_CHANNELS);
			}

				/* clip the sound size */
//			for(i=0;i<dsp_server_fragsize;i++)
			for(i=0;i<dsp_server_samples *dsp_server_channels;i++)
			{
				if (dsp_server_type==2) { //8 bit uncompressed
					final[i] = (premix[i]>255)?255:(premix[i]<-256?0:(premix[i]>>1)+128);
				}
				else { // 16 bit
						//Assuming lsb audio hardware:
//					int final_tmp=(premix[i]>32767)?32767:(premix[i]<-32768?-32768:premix[i]);
					int final_tmp=premix[i];
					final[i*2]= final_tmp &0xff;
					final[i*2+1] = (final_tmp>>8) &0xff;
				}
			}

				/* check end of files -> if so, stop/repeat it (i.e. repeat's are only done per fragment, up to a fragment-1 samples silence betweeen; change?) */
			for(i=playnum-1; i>=0; i--) {
				if (position[i]>=sound_size[playing[i]]) {
					DEB1(fprt(stderr, "finished playing sound %d (queue no %d)\n", playing[i], i));
					DEB1(fprt(stderr, "  it was at position %d\n", position[i]));
					if (repeat[i]) {
						position[i]=0;
						DEB1(fprt(stderr, "  repeating it"));
					}
					else {
						if  ( i<playnum-1 ) {
							if (i<last_sound_stack)
								last_sound_stack--;
							else if (i==last_sound_stack)
								last_sound_stack = -1;
							memmove(repeat+i,
									repeat+i+1, (playnum-i-1)*sizeof(int));
							memmove(playing+i,
									playing+i+1, (playnum-i-1)*sizeof(int));
							memmove(position+i,
									position+i+1, (playnum-i-1)*sizeof(int));
							memmove(sound_x+i,
									sound_x+i+1, (playnum-i-1)*sizeof(int));
							memmove(sound_dx+i,
									sound_dx+i+1, (playnum-i-1)*sizeof(int));
						}
						playnum--;
					}
				}
			}


			for(i=channel_playnum-1; i>=0; i--) {
				int  j=MAX_DYNAMIC_CHANNELS+channel_used[i];
				if (position[j]>=sound_size[playing[j]]) {
					DEB1(fprt(stderr, "finished playing sound %d (channel no %d)\n", playing[j], channel_used[i]));
					DEB1(fprt(stderr, "  it was at position %d\n", position[j]));
					if (repeat[j]) {
						position[j]=0;
						DEB1(fprt(stderr, "  repeating it"));
					}
					else {
						if  ( i<channel_playnum-1 ) {
							if (i<last_sound_channel)
								last_sound_channel--;
							else if (i==last_sound_channel)
								last_sound_channel=-1;
							memmove(channel_used+i,
									channel_used+i+1, (channel_playnum-i-1)*sizeof(int));
						}
						channel_playnum--;
						playing[j]=-1; /* used as a flag to check if a sound is playin on the channel, should use a separate status_flag[MAX_CHANNELS] byte/int  */
						if (do_callback) {
							status_send_byte(GSI_CALLBACK_CHANNEL_END);
							status_send_byte(channel_used[i]);
							DEB1(fprintf(stderr, "callback in queue\n"));
						}
					}
				}
			}
		} else {
				/* No sounds to play so fill the buffer with silence */
			if (dsp_server_type==GSI_8BIT) { //8 bit uncompressed unsigned   
				memset(final, 128, dsp_server_fragsize);
			}
			else { //16 bit
				memset(final, 0, dsp_server_fragsize);
			}
		}
		write (dsp, final, dsp_server_fragsize);
	}
} // ^ sndserver





///////////// calculate one fragment of a given sound ///////////////
void play_fragment(int i) {
	int  j;
	int  x; //for stereo position
//	stereo_shift=channels-sound_channels[i];
	char *sample_position;
	int sample_length;
	int conversion;
#if 1 /* Doppler version */
//Doppler stuff:
	int  sound_conv;
	int  sound_conv2;
	int  f;

	if (sound_x[i]* sound_dx[i]<0)
		f=(256-sound_doppler[i]); //should use z also to correct this factor.
	else
		f=(256+sound_doppler[i]);
DEB1(fprintf(stderr, "doppler, f=%d\n", f));
#if 0
	sound_conv = ( ((sound_speed[playing[i]]<<8)/dsp_server_speed)*256) /(256+sound_doppler[i]);

	sound_conv2= ( ((dsp_server_speed<<8)/sound_speed[playing[i]])*256) /(256+sound_doppler[i])) /256;
#else /* wrong order? */
	sound_conv = ( ((sound_speed[playing[i]]<<8)/dsp_server_speed)*256) /f;

	sound_conv2= ( ((dsp_server_speed<<8)/sound_speed[playing[i]])* f) /256;
#endif

#else
	int sound_conv=sound_conversion[playing[i]];
	int sound_conv2=(dsp_server_speed<<8)/sound_speed[playing[i]];
#endif

		/*char * */ sample_position=sound_buffer[playing[i]]+position[i];
/*	int sample_length;
	int conversion;
	*/
	DEB2(fprt(stderr, "sound %d, sound conv=%d, sound_conv2=%d\n", playing[i], sound_conv, sound_conv2));

//	int sample_length=min(dsp_server_fragsize, (((sound_size[playing[i]]-position[i])*sound_conv2)>>8));

//	int sample_length=min(dsp_server_samples, (((sound_size[playing[i]]-position[i])*sound_conv2)>>8) * (dsp_server_bps*dsp_server_channels) );

	sample_length=min(dsp_server_samples, (((sound_size[playing[i]]-position[i])*sound_conv2)>>8) / (sound_channels[playing[i]]* ((sound_type[playing[i]] +1)/2) ));

	conversion=0;
	conversion |= (dsp_server_type & 1);
	conversion |= (dsp_server_channels-1)<<1;
	conversion |= (sound_type[playing[i]] & 1)<<2;
	conversion |= (sound_channels[playing[i]]-1)<<3;


#define S16_M8   (3<<2)
#define S16_M16 ((3<<2) | 1)
#define S16_S8  ((3<<2) | 2)
#define S16_S16 ((3<<2) | 3)

#define S8_M8   (2<<2)
#define S8_M16  ((2<<2) | 1)
#define S8_S8   ((2<<2) | 2)
#define S8_S16  ((2<<2) | 3)

#define M16_M8   (1<<2)
#define M16_M16 ((1<<2) | 1)
#define M16_S8  ((1<<2) | 2)
#define M16_S16 ((1<<2) | 3)

#define M8_M8   (0<<2)
#define M8_M16  ((0<<2) | 1)
#define M8_S8   ((0<<2) | 2)
#define M8_S16  ((0<<2) | 3)

	switch(conversion) {
			// Possible change: always 16 bit premix, hence, with 8 bit
			// samples do <<8, and in the final mixing, decide if >>8 is
			// needed.

//-------------------------- S16 sound -------------------------
	case S16_M8: /* stereo 16 bit sound to mono 8 bit DSP */
		DEB2(fprt(stderr, "S16_M8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int16 b1, b2;
			b1  = ((byte) (*(sample_position+(((sound_conv*j)>>6) & ~3)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>6) & ~3)+1);
			b2  = ((byte) (*(sample_position+(((sound_conv*j)>>6) & ~3)+2))) <<8;
			b2 |=  (byte) *(sample_position+(((sound_conv*j)>>6) & ~3)+3);
			premix[j]+= (((int) b1+ (int) b2)>>(1+8));
		}
		position[i] += ((dsp_server_samples*sound_conv)>>6) & ~3;

		break;

	case S16_M16: /* stereo 16 bit sound to mono 16 bit DSP */
		DEB2(fprt(stderr, "S16_M16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int16 b1, b2;
			b1  = ((int) *(sample_position+(((sound_conv*j)>>6) & ~3)+0))<<8;
			b1 |= (byte) *(sample_position+(((sound_conv*j)>>6) & ~3)+1);
			b2  = ((int) *(sample_position+(((sound_conv*j)>>6) & ~3)+2))<<8;
			b2 |= (byte) *(sample_position+(((sound_conv*j)>>6) & ~3)+3);
			premix[j]+= (b1+b2)>>(1+0);
		}
		position[i] += ((dsp_server_samples*sound_conv)>>6) & ~3;
		break;

	case S16_S8: /* stereo 16 bit sound to stereo 8 bit DSP */
		DEB2(fprt(stderr, "S16_S8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length*2;j+=2) {
			int b1, b2;
			b1  = ((int) *(sample_position+(((sound_conv*j)>>7) & ~3)+0))<<8;
			b1 |= (byte) *(sample_position+(((sound_conv*j)>>7) & ~3)+1);
			b2  = ((int) *(sample_position+(((sound_conv*j)>>7) & ~3)+2))<<8;
			b2 |= (byte) *(sample_position+(((sound_conv*j)>>7) & ~3)+3);
			premix[j]   += (b1)>>(0+8);
			premix[j+1] += (b2)>>(0+8);
		}
		position[i] += ((dsp_server_samples*sound_conv)>>6) & ~3;
		break;

	case S16_S16: /* stereo 16 bit sound to stereo 16 bit DSP */
		DEB2(fprt(stderr, "S16_S16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length*2;j+=2) {
			int b1, b2;
			b1  = ((int) *(sample_position+(((sound_conv*j)>>7) & ~3)+0))<<8;
			b1 |= (byte) *(sample_position+(((sound_conv*j)>>7) & ~3)+1);
			b2  = ((int) *(sample_position+(((sound_conv*j)>>7) & ~3)+2))<<8;
			b2 |= (byte) *(sample_position+(((sound_conv*j)>>7) & ~3)+3);
			premix[j]   += (b1)>>(0);
			premix[j+1] += (b2)>>(0);
		}
		position[i] += ((dsp_server_samples*sound_conv)>>6) & ~3;
		break;


//-------------------------- S8 -------------------------
	case S8_M8:  /* stereo 8 bit sound to mono 8 bit DSP */
		DEB2(fprt(stderr, "S8_M8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int b1, b2;
			b1  = (int) *(sample_position+(((sound_conv*j)>>7)&~1)+0);
			b2  = (int) *(sample_position+(((sound_conv*j)>>7)&~1)+1);
			premix[j]+= (b1+b2)>>(1+0);
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case S8_M16:  /* stereo 8 bit sound to mono 16 bit DSP */
		DEB2(fprt(stderr, "S8_M16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int16 b1, b2;
			b1  = (int16) *(sample_position+(((sound_conv*j)>>7)&~1)+0);
			b2  = (int16) *(sample_position+(((sound_conv*j)>>7)&~1)+1);
			premix[j]+= ((int) b1+ (int) b2)<<7;
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case S8_S8:  /* stereo 8 bit sound to stereo 8 bit DSP */
		DEB2(fprt(stderr, "S8_S8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length*2;j+=2) {
			int16 b1, b2;
			b1  = (int16) *(sample_position+(((sound_conv*j)>>8)&~1)+0);
			b2  = (int16) *(sample_position+(((sound_conv*j)>>8)&~1)+1);
			premix[j]  += b1;
			premix[j+1]+= b2;
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case S8_S16:  /* stereo 8 bit sound to stereo 16 bit DSP */
		DEB2(fprt(stderr, "S8_S16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length*2;j+=2) {
			int16 b1, b2;
			b1  = (int16) *(sample_position+(((sound_conv*j)>>8)&~1)+0);
			b2  = (int16) *(sample_position+(((sound_conv*j)>>8)&~1)+1);
			premix[j]  += b1<<8;
			premix[j+1]+= b2<<8;
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;


//-------------------------- M16 -------------------------
	case M16_M16:  /* mono 16 bit sound to mono 16 bit DSP */
		DEB2(fprt(stderr, "M16_M16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>7) & ~1)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>7) & ~1)+1);
			premix[j]   += (int) b1;
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case M16_M8:  /* mono 16 bit sound to mono 8 bit DSP */
		DEB2(fprt(stderr, "M16_M8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		for(j=0;j<sample_length;j++) {
			int16 b1;
			b1  = ((int) *(sample_position+(((sound_conv*j)>>7) & ~1)+0))<<8;
			b1 |= (byte) *(sample_position+(((sound_conv*j)>>7) & ~1)+1);
			premix[j]   += (b1)>>(0+8);
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case M16_S8:  /* mono 16 bit sound to stereo 8 bit DSP */
		DEB2(fprt(stderr, "M16_S8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));

			//Calc sound volume for mono:
		sound_x[i] +=sound_dx[i]/*+sound_move_all*/;
		x=sound_x[i]-observer_x;

		sound_volume_L=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x+EAR_POS)*(x+EAR_POS));
		sound_volume_R=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x-EAR_POS)*(x-EAR_POS));

		for(j=0;j<sample_length*2;j+=2) {
#ifdef NO_EFFECTS
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>8) & ~1)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>8) & ~1)+1);
			premix[j]   += (int) b1 >>(0+8);
			premix[j+1] += (int) b1 >>(0+8);
#else
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>8) & ~1)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>8) & ~1)+1);
			premix[j]   += (((int) b1) * sound_volume_L) >>(0+8+8);
			premix[j+1] += (((int) b1) * sound_volume_R) >>(0+8+8);
#endif
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;

	case M16_S16:  /* mono 16 bit sound to stereo 16 bit DSP */
		DEB2(fprt(stderr, "M16_S16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));

			//Calc sound volume for mono:
		sound_x[i] +=sound_dx[i]/*+sound_move_all*/;
		x=sound_x[i]-observer_x;

		sound_volume_L=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x+EAR_POS)*(x+EAR_POS));
		sound_volume_R=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x-EAR_POS)*(x-EAR_POS));


		if (sound_conv==256) {
			for(j=0;j<sample_length*2;j+=2) {
				int16 b1, b2;
				b1  = ( (int) (*(sample_position+((j) & ~1)+0)) ) <<8;
				b1 |=   (byte) *(sample_position+((j) & ~1)+1);
				premix[j]   += (((int) b1) *sound_volume_L)>>8;
				premix[j+1] += (((int) b1) *sound_volume_R)>>8;
			}
		}
		else {
		for(j=0;j<sample_length*2;j+=2) {
#ifdef NO_EFFECTS
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>8) & ~1)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>8) & ~1)+1);
			premix[j]   += (int) b1;
			premix[j+1] += (int) b1;
#else
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>8) & ~1)+0))) <<8;
			b1 |=  (byte) *(sample_position+(((sound_conv*j)>>8) & ~1)+1);
			premix[j]   += (((int) b1) *sound_volume_L)>>8;
			premix[j+1] += (((int) b1) *sound_volume_R)>>8;
#endif
		}
		}
		position[i] += ((dsp_server_samples*sound_conv)>>7) & ~1;
		break;


//-------------------------- M8 -------------------------
	case M8_M8:  /* mono 8 bit sound to mono 8 bit DSP */
		DEB2(fprt(stderr, "M8_M8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:


#if 0
		{
			int oversample=0;
			int oversample_inc=0;
		}
#endif
		for(j=0;j<sample_length;j++) {
//		premix[j] += *(sample_position+((sound_conv*j)>>8)); //standard

//		premix[j] += ((int)*(sample_position+((sound_conv*j)>>8)) +(int) *(sample_position+((sound_conv*j)>>8)+1) ) /2 ; //doesn't work well
//		premix[j] += (( ((sound_conv*j)&255) * ((int)*(sample_position+((sound_conv*j)>>8)) - (int) *(sample_position+((sound_conv*j)>>8)+1) ) )>>8) + (int)*(sample_position+((sound_conv*j)>>8));
#if 1
//oversample code v2
//Standard interpolation !!this code gives good results!!
			{
				int delta=((sound_conv*j)&255);
				int premix_plus = ( (256-delta) * ((int)*(sample_position+((sound_conv*j)>>8))) + delta* (int)*(sample_position+((sound_conv*j)>>8)+1) )>>8;
				premix[j] +=premix_plus;

//fprintf(stderr, "j=%d, premix+=%d, l=%d, r=%d, frac=%d\n", j, premix_plus, *(sample_position+((sound_conv*j)>>8)), *(sample_position+((sound_conv*j)>>8)+1), delta);
			}
#endif
#if 0
//oversample v3, unfinished
			{
				int new_item=*(sample_position+((sound_conv*j)>>8));
				sample_factor=(float) sound_conv /256.0;

				prev_item = new_item;
				new_item = oversample;
				oversample += oversample_inc;
				if (sample_position+sample_factor > last_offset + 1) {
					oversample_inc = (prev_item-oversample)*sample_factor;
				}
				premix[i] += ((new_item*volume[playing[i]])>>8); //software mixer added too.
			}
#endif

		}
		position[i] += ((dsp_server_samples*sound_conv)>>8);
		break;

	case M8_M16:  /* mono 8 bit sound to mono 16 bit DSP */
		DEB2(fprt(stderr, "M8_M16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));
		sound_volume_L=sound_volume[i]; //not yet used, change code below:

		if (sound_conv==256) { //no doppler or speed conversion, faster code
			for(j=0;j<sample_length;j++) {
				premix[j] +=(*(sample_position+(j))) <<8;
			}
		}
		else {
			for(j=0;j<sample_length;j++) {
				premix[j] +=(*(sample_position+((sound_conv*j)>>8))) <<8;
			}
		}
		position[i] += ((dsp_server_samples*sound_conv)>>8);
		break;

	case M8_S8:  /* mono 8 bit sound to stereo 8 bit DSP */
		DEB2(fprt(stderr, "M8_S8, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));

			//Calc sound volume for mono:
		sound_x[i] +=(sound_dx[i]/*+sound_move_all*/);
		x=sound_x[i]-observer_x;
		DEB2(fprt(stderr, "x=%d: stereo_pos=%d, stereo_speed=%d, observer_x=%d\n", x, sound_x[i], sound_dx[i], observer_x));

		sound_volume_L=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x+EAR_POS)*(x+EAR_POS));
		sound_volume_R=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x-EAR_POS)*(x-EAR_POS));

		DEB2(fprt(stderr, "vol L,R=%d, %d\n", sound_volume_L, sound_volume_R));

		if (sound_conv==256) { //no doppler or speed conversion, faster code
			for(j=0;j<sample_length*2;j+=2) {
				premix[j]   += ( ((int) *(sample_position+(j>>1)) ) * sound_volume_L)>>8;
				premix[j+1] += ( ((int) *(sample_position+(j>>1)) ) * sound_volume_R)>>8;
			}
		}
		else {
		for(j=0;j<sample_length*2;j+=2) {
#ifdef NO_EFFECTS
			premix[j]   += *(sample_position+((sound_conv*j)>>/*8*/ 9));
			premix[j+1] += *(sample_position+((sound_conv*j)>>/*8*/ 9));
#else
			premix[j]   += (((int) *(sample_position+((sound_conv*j)>>/*8*/ 9))) * sound_volume_L)>>8;
			premix[j+1] += (((int) *(sample_position+((sound_conv*j)>>/*8*/ 9))) * sound_volume_R)>>8;
#endif
		}
		}
		position[i] += ((dsp_server_samples*sound_conv)>>8);
		break;

	case M8_S16:  /* mono 8 bit sound to stereo 16 bit DSP */
		DEB2(fprt(stderr, "M8_S16, sound %d (snd file %d), sample length=%d\n", i, playing[i], sample_length));
		DEB2(fprt(stderr, "sample_position %d\n", position[i]));

			//Calc sound volume for mono:
		sound_x[i] +=sound_dx[i]/*+sound_move_all*/;
		x=sound_x[i]-observer_x;

		sound_volume_L=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x+EAR_POS)*(x+EAR_POS));
		sound_volume_R=(sound_volume[i]*(EAR_DIST*EAR_DIST)) /(EAR_DIST*EAR_DIST+(x-EAR_POS)*(x-EAR_POS));

		if (sound_conv==256) {
			for(j=0;j<sample_length*2;j+=2) {
				int16 b1, b2;
				b1  = ((int) (*(sample_position+(j>>1) )));
				premix[j]   += ((int) b1) * sound_volume_L;
				premix[j+1] += ((int) b1) * sound_volume_R;
			}
		}
		else {
		for(j=0;j<sample_length*2;j+=2) {
#ifdef NO_EFFECTS
			premix[j]   += ( (int) *(sample_position+((sound_conv*j)>>9))) <<8;
			premix[j+1] += ( (int) *(sample_position+((sound_conv*j)>>9))) <<8;
#else
			int16 b1, b2;
			b1  = ((int) (*(sample_position+(((sound_conv*j)>>9) ))) );
			premix[j]   += ((int) b1) * sound_volume_L;
			premix[j+1] += ((int) b1) * sound_volume_R;

#endif
		}
		}
		position[i] += ((dsp_server_samples*sound_conv)>>8);
		break;
	}
}













// -- + -------------------------------------------------------- +
// Utilities:

static int sounds_file_loaded=False;
//unload a sounds file and all the associated sounds.
void unload_sounds_file() {
	int  i;

	if (sounds_file_loaded)
		for (i=0; i<256; i++) {
			if (sound_from_list[i]) {
					//We only free sounds that were not loaded separately.
//				if (sound_buffer[i]!=NULL) {
//					free(sound_buffer[i]);
//					sound_buffer[i]=NULL;
//					sound_size[slot] = 0; /* used to test if sound was loaded in main loop */
//				}
				unload_sound(i);
				free(SOUNDFILENAME[i]);
			}
			sounds_file_loaded=False;
		}
}

void unload_sound(int slot) {
DEB1(fprintf(stderr, "unload %d\n", slot));
	if (slot < /*max_slot*/ MAX_SOUNDS) {
		if (sound_buffer[slot]!=NULL) {
			free(sound_buffer[slot]);
			sound_buffer[slot]=NULL;
			sound_size[slot] = 0; /* used to test if sound was loaded in main loop */
		}
	}
DEB1(fprintf(stderr, "done\n"));
}

//Load the file listing all sounds used, and allocs memory for the complete
//paths. The last line is the number of sounds
int init_sounds_file(char *sndsfile_name) {
	int  i;
	char s[1024]; /* conversion string, 128 should be enough */
	char s2[1024];
	FILE *fp;


		//Unload old file.
	if (sounds_file_loaded) { unload_sounds_file(); sounds_file_loaded=False; }

	if (sndsfile_name==NULL) return -1;
	if (strlen(sndsfile_name)==0) return -1;

	strcpy(s, sndsfile_name);

	fp = fopen(s, "r");
	if( fp == 0 ) {
		fprintf (stderr, "%s: The sounds file %s could not be opened\n", server_name, sndsfile_name);
		return -1;
	}
	for (i=0; i<256; i++) {
		fgets(s2, 127, fp); s2[127]=0;
		if ( feof(fp) ) break;
		if ( strlen(s2)==0 ) break; /* empty entry */
		s2[(int) strlen(s2) -1]=0; /* get rid of the linefeed */
		strcpy(s, sounds_dir);

#if 1
		if ( s[(int)strlen(s) - 1] != '/' )
			strcat (s, "/");
		if ( s2[0] == '/' )
			strcpy (s, s2); /* absolute path */
		else
			strcat (s, s2); /* relative from the sounddir */
#else
		if ( s[(int)strlen(s) - 1] == '/' )
			strcat (s, s2+1);
		else
			strcat (s, s2);
#endif
//	if (SOUNDFILENAME[i]==NULL)
		SOUNDFILENAME[i] = (char *) malloc ((int)strlen (s)+1);

		strcpy (SOUNDFILENAME[i],s);

		sound_buffer[i]=NULL;
		sound_size[i]=0;
	}
	fclose(fp);
	num_sounds=max(num_sounds, i); /* use previous 'num_sounds' */
DEB1(fprt(stderr, "num sounds=%d\n", num_sounds));
	sounds_file_loaded=True;
	DEB1(fprt(stderr, "sounds file loaded\n"));
	return 0;
}


///////////// Read all sound files from the list from disk ///////////////
void read_soundsfile_sounds() {
		/* load all sounds ( to prevent hickups when a sound is first used ) */
	int i;
	DEB1(fprt(stderr, "GSI_SERVER: read_soundsfile_sounds\n"));
	for (i=0; i<num_sounds; i++) {
		if (!read_sound(i, SOUNDFILENAME[i]))
			sound_from_list[i]=1;
	}
	sounds_loaded=1;
}


///////////// Read a sound file from disk ///////////////
int read_sound(int slot, char *sound_file) {
	int i,fd,size, data_start;
	struct stat fd_stat;

	if ( slot>=num_sounds ) {
		fprintf (stderr, "%s: invalid sound number %d\n", server_name, slot);
		return(-1);
	}
	DEB1(fprt(stderr, "loading sound %d: %s\n",slot,sound_file));

	fd = open(sound_file, O_RDONLY);
	if( fd <= 0 )  {
		fprintf (stderr, "%s: The sound %s could not be opened\n", server_name, sound_file);
		sound_size[slot]=-1;
		return(-1);
	}

		//Check that the file is a regular file (not a dir)
	fstat(fd, &fd_stat);
	if (S_ISREG(fd_stat.st_mode)) {
		int  name_len=strlen(sound_file);
		char *name_end=sound_file+name_len;

		if (!strcmp(name_end-4,".raw")) {
			sound_speed[slot]=dsp_server_speed;
			sound_type[slot]=dsp_server_type;
			sound_channels[slot]=1; /* dsp_server_channels */
			size=lseek(fd,0,SEEK_END);
			data_start=0;
		}
		else
			if (!strcmp(name_end-3,".au") ||
				!strcmp(name_end-4,".snd")) {
				lseek(fd,4,SEEK_SET);
				data_start=int_msb(fd);
				size=int_msb(fd);
				sound_type[slot]=int_msb(fd);
				sound_speed[slot]=int_msb(fd);
				sound_channels[slot]=int_msb(fd);
				DEB1(fprt(stderr, "type=%d, speed=%d, channels=%d\n", sound_type[slot], sound_speed[slot], sound_channels[slot]));
			}
			else {
				fprintf(stderr, "%s: unrecognized file format of %s\n", server_name, sound_file);
			}

//		if (sound_buffer[slot] !=NULL)
//			free(sound_buffer[slot]); /* unload a previous sound */
		unload_sound(slot);

		sound_buffer[slot]=(sbyte *) malloc(/*sound_size[slot]*/size);
		if(sound_buffer[slot]==NULL) {
			fprintf(stderr,"%s: couldn't allocate memory for sound\n", server_name);
//			sound_size[slot]=-1; //see unload_sound, must be and is set to 0.
			close(fd);
			return(-1);
		}
		sound_size[slot]=size;

		lseek(fd,data_start,SEEK_SET);
		read(fd,sound_buffer[slot],size);
	}
	else {
		DEB1(fprt(stderr, " %d not a regular file\n", slot));
		close(fd); return -1;
	}
	close(fd);
	if (sound_type[slot]==2) {
			/* unsigned 8 bit to signed 8 bit */
		for(i=0; i<size; i++) sound_buffer[slot][i]^=0x80;
	}

	DEB1(fprt(stderr, " -> loaded, %d bytes\n",size));
	DEB1(fprt(stderr, "    speed: %d\n",sound_speed[slot]));
		//Conversion factors ..
	set_sound_conversion_of(slot);

	return(0);
}


void init_sample(int slot, int speed, int type, int channels, int size) {
	int  i;

	sound_type[slot] = type;
	sound_speed[slot] = speed;
	sound_channels[slot] = channels;
	sound_size[slot] = size;
	DEB1(fprt(stderr, "  GSI_SERVER: set %d to: type=%d, speed=%d, channels=%d (% d bytes)\n", slot, type, speed, channels, size));
	if (sound_type[slot]==2) {
			/* unsigned 8 bit to signed 8 bit */
		for(i=0; i<size; i++) sound_buffer[slot][i] ^=0x80;
	}

		//Conversion factors ..
	set_sound_conversion_of(slot);
}

///////////// Read a msb-int from a file ///////////////
int int_msb(int fd) {
	byte file_byte;
	int  i,int_return=0;

	for (i=3; i>=0; i--) {
		read(fd, &file_byte, 1);
		int_return |= ((int)file_byte << (i*8));
	}
	return int_return;
}


///////////// Calulate speed conversion factors ///////////////
//This stuff must be replaced with a run length algorithm
void set_sound_conversion() {
	int i;
	DEB1(fprt(stderr, "GSI_SERVER: set soundconversion\n"));
	for (i=0; i< num_sounds; i++) {
		if (sound_size[i]>0) { /* sound was really loaded! */
			sound_conversion[i]=(sound_speed[i]<<8)/dsp_server_speed;
			sound_conversion2[i]=(dsp_server_speed<<8)/sound_speed[i];
		}
	}
		//stereo -> mono: sound_conversion[i] *=2;
		//mono -> stereo: sound_conversion[i] /=2;
		//8bit -> 16 bit: H -> H,0
		//16bit -> 8 bit: H,L -> H
}
void set_sound_conversion_of(int slot) {
	if (sound_size[slot]>0) {
		sound_conversion[slot]=(sound_speed[slot]<<8)/dsp_server_speed;
		sound_conversion2[slot]=(dsp_server_speed<<8)/sound_speed[slot];
	}
}

/* Make a integer sinus table, values are 65536*sinus(2 pi* i/1024) */
void make_sinus() {
	int  i;

	for (i=0; i<1024; i++) {
		sinus_tabel[i]= (int) (sin((PI*2.0*i)/1024.0) *65536.0 +.5);
	}
}

///////////// Pipe functions ///////////////
void get_block(byte *data, int len) {
	fd_set read_fd_sel;
	int i=0, j=0;

	DEB1(fprt(stderr, "GSI_SERVER: sound_block: receiving a block of %d bytes\n", len));
	while((j=read (STDIN_FILENO, data+i, len-i)) <len-i) {
		if (j>0) i +=j;
		FD_ZERO(&read_fd_sel);
		FD_SET(STDIN_FILENO, &read_fd_sel);
		select(STDIN_FILENO+1, &read_fd_sel, NULL, NULL, NULL);
	}
}

byte get_byte() {
	fd_set read_fd_sel;
	byte pipe_b;
	while ( read(STDIN_FILENO, &pipe_b, sizeof(byte))<1 ) {
			FD_ZERO(&read_fd_sel);
			FD_SET(STDIN_FILENO, &read_fd_sel);
			select(STDIN_FILENO+1, &read_fd_sel, NULL, NULL, NULL);
	}

	return pipe_b;
}
int16 get_int16() { //get little endian int16
//	int16 data;
	byte b1, b2;

	b1=get_byte();
	b2=get_byte();
	return b1 | (b2<<8);
}
int32 get_int32() { //get little endian int32
//	int32 data;
	byte b1,b2,b3,b4;

	b1=get_byte();
	b2=get_byte();
	b3=get_byte();
	b4=get_byte();
	return b1 | (b2<<8) | (b3<<16) | (b4<<24);
}


char *get_string() {
	int  i, retval;
	int  size=256;
	char *tmp_string;
	char string_c;
	int  string_len=0;
	char *tmp_2;

	tmp_string=(char *) malloc(size);

	for (;;) {
		fd_set read_fd;
		FD_ZERO(&read_fd);
		FD_SET(STDIN_FILENO, &read_fd);
			/*int */retval = select(STDIN_FILENO+1, &read_fd, NULL, NULL, NULL);
		i = read(STDIN_FILENO, &string_c, sizeof(string_c));
		if ( i>0 ) {
			tmp_string[string_len++]=string_c;
			if (string_c==0) break;
			if (string_len==size) {
				size +=32; tmp_2=(char *) malloc(size);
				strncpy(tmp_2, tmp_string, size-32); 
				free(tmp_string);
				tmp_string=tmp_2;
			}
		}
	}
	return tmp_string;
}

//Calback
void status_send_byte(int data) {
	byte b=data;

	while (status_queue_len>=MAX_STATUS_QUEUE_LEN) status_flush();
	status_queue[status_queue_len++]=b;
}
//soft flush (flush as many bytes as possible without blocking)
void status_flush() {
	int j;

	if (status_queue_len==0) return;
	DEB1(fprt(stderr, "GSI_SERVER: status flush (queue len=%d)\n", status_queue_len));
	j=write (STDOUT_FILENO, status_queue, status_queue_len);
	if (j>0) {
		status_queue_len -=j;
		if (status_queue_len) memmove(status_queue, status_queue+j, status_queue_len);
	}
	DEB1(fprt(stderr, "          : done (queue len=%d)\n", status_queue_len));
}

///////////// Signal handlers ///////////////
/* SIGTERM */
void quit (int i) {
	DEB1(fprt(stderr, "quit %s\n", server_name));
	exit (0);
}


#ifdef SEGFAULT_HANDLER
/* SIGSEGV, only for debugging */
void segfault(int msg) {
	fprintf(stderr, "\n%s: segfault\n", server_name);
	exit(-1);
}
#endif




/* Note on mixing: 
   What about overflow? Isn't it more natural for a fixed range system to use:
   1. Log's so that 255+255=255, 10+10 approx 20, 10+250=252 etc.
      This needs at least 2 256 b tables:
	     premix[j]+=logtable[*(sample[+j)];
	 Then after all adds:
	     premix[j]=premix[j]/num_playing;
	     premix[j]=exptable[premix[j];
   2. Fast approximation: just use max: loudness=max(loudness1, ..., loudness n)
	     premix[j]=((*(sample+j)) >premix[j] ? *(sample+j) : premix[j])
*/

/* ^ gsi_server.c */
